==> desktopbsd-tools-1.0.1.tar.bz2: desktopbsd-tools/libdesktopbsd/pkgupgradeengine.cpp <==
/*
 * This file is part of DesktopBSD, see the README file.
 *
 * Author: Peter Hofer <hofer.p@gmail.com>
 *			(C) 2004, 2005
 *
 * Copyright: BSD, see the COPYING file included in this distribution
 *
 */

#include "pkgupgradeengine.h"

#include <sys/utsname.h>

#include <sysctl.h>
#include <unixprocess.h>
#include <dbsdsettings.h>

#define	PORTUPGRADE_CMD	"/usr/local/sbin/portupgrade"
#define	PACKAGES_DIR		"/usr/ports/packages/All"


extern "C" int getosreldate(void);


PkgUpgradeEngine::PkgUpgradeEngine() {
	/* answerY = false; */
	keepGoing = false;
	installNewPkgs = false;
	recursion = NoRecursion;
	packageUse = UsePorts;
}


/** Loads the "packages" settings of the DesktopBSD configuration
  * using DBSDSettings.
  */
void  PkgUpgradeEngine::loadSettings() {
	DBSDSettings  s;

	s.beginGroup("/packages");
	{
		int  rec = 0;
		if(s.readBoolEntry("upward_recursive", false))
			rec |= UpwardRecursive;

		if(s.readBoolEntry("recursive", false))
			rec |= Recursive;

		setRecursion(rec);
		setKeepGoing(s.readBoolEntry("keep_going", false));
		setPackageUse((PackageUse)  s.readNumEntry("package_use", 2));
		addMakeArgs(s.readListEntry("make_args"));
		addMakeEnv(s.readListEntry("make_env"));
	}
}



/** Returns a QProcess that applies the specified
  * actions on the given list of packages or NULL
  * if not packages for upgrading were specified.
  *
  * The returned QProcess must be deleted after use.
  */
UnixProcess * PkgUpgradeEngine::getProcess() {
	/* Stop here when no packages are specified */
	if(packages.isEmpty())
		return  NULL;


	UnixProcess  *process = new UnixProcess(QString(PORTUPGRADE_CMD));
	process->setWorkingDirectory(QDir(PACKAGES_DIR));

	if(recursion & Recursive)
		process->addArgument("-r");
	if(recursion & UpwardRecursive)
		process->addArgument("-R");

	if(installNewPkgs)
		process->addArgument("-N");

	if(keepGoing)
		process->addArgument("-k");

	switch(packageUse) {
		case UsePackages:
			process->addArgument("-P");
			break;

		case UsePackagesOnly:
			process->addArgument("-PP");
			break;

		default: break;
	}

	if(!makeArgs.isEmpty()) {
		process->addArgument("-m");
		process->addArgument(makeArgs.join(" "));
	}

	if(!makeEnv.isEmpty()) {
		process->addArgument("-M");
		process->addArgument(makeEnv.join(" "));
	}

	Package  *pkg = packages.first();
	while(pkg != NULL) {
		process->addArgument(pkg->getID());

		pkg = packages.next();
	}

	process->setEnvironment(getEnvironment());
	return  process;
}


QString PkgUpgradeEngine::readDefaultRepository() {
	const struct {
		int from;     /* Lowest version number to match */
		int to;      /* Highest version number to match */
		const char *repository;  /* Directory it lives in */
	} releases[] = {
		{ 410000, 410000, "4.1-release" },
		{ 420000, 420000, "4.2-release" },
		{ 430000, 430000, "4.3-release" },
		{ 440000, 440000, "4.4-release" },
		{ 450000, 450000, "4.5-release" },
		{ 460000, 460001, "4.6-release" },
		{ 460002, 460099, "4.6.2-release" },
		{ 470000, 470099, "4.7-release" },
		{ 480000, 480099, "4.8-release" },
		{ 490000, 490099, "4.9-release" },
		{ 491000, 491099, "4.10-release" },
		{ 492000, 492099, "4.11-release" },
		{ 500000, 500099, "5.0-release" },
		{ 501000, 501099, "5.1-release" },
		{ 502000, 502009, "5.2-release" },
		{ 502010, 502099, "5.2.1-release" },
		{ 503000, 503099, "5.3-release" },
		{ 504000, 504099, "5.4-release" },
		{ 600000, 600099, "6.0-release" },
		{ 300000, 399000, "3-stable" },
		{ 400000, 499000, "4-stable" },
		{ 502100, 502128, "5-current" },
		{ 503100, 599000, "5-stable" },
		{ 600100, 699000, "6-stable" },
		{ 0, 9999999, "current" },
		{ 0, 0, NULL }
	};


	int release = getosreldate();
	for(int i = 0; releases[i].repository != NULL; ++i)
		if(release >= releases[i].from && release <= releases[i].to)
			return releases[i].repository;

	return "";
}


QStringList PkgUpgradeEngine::getEnvironment() {
	/* Find out architecture */
	struct utsname u;
	uname(&u);

	/* Server package directory */
	QString packageDirectory = DBSDSettings().readEntry("/packages/package_directory");
	
	QString site = "PACKAGESITE=";
	if(packageDirectory.startsWith("ftp://") || packageDirectory.startsWith("http://"))
		site += packageDirectory;
	else {
		site += QString("ftp://ftp.freebsd.org/pub/FreeBSD/ports/%1/packages-%2/Latest/")
						.arg(u.machine)
						.arg(packageDirectory.isEmpty() ? readDefaultRepository() : packageDirectory);
	}
				  
	return QStringList(site);
}

==> desktopbsd-tools-1.6.tar.bz2: desktopbsd-tools/libdesktopbsd/pkgupgradeengine.cpp <==
/*
 * This file is part of DesktopBSD, see the README file.
 *
 * Author: Peter Hofer <ph@desktopbsd.net>
 *			(C) 2004, 2005
 *
 * Copyright: BSD, see the COPYING file included in this distribution
 *
 */

#include "pkgupgradeengine.h"

#include <sys/utsname.h>

#include <sysctl.h>
#include <unixprocess.h>
#include <packagesettings.h>
#include <pkgconstants.h>

extern "C" int getosreldate(void);


PkgUpgradeEngine::PkgUpgradeEngine() {
	/* answerY = false; */
	keepGoing = false;
	installNewPkgs = false;
	recursion = NoRecursion;
	packageUse = UsePorts;
}


/** Loads the "packages" settings of the DesktopBSD configuration
  * using DBSDSettings.
  */
void  PkgUpgradeEngine::loadSettings() {
	PackageSettings s;

	int  rec = 0;
	if(s.getUpwardRecursive())
		rec |= UpwardRecursive;

	if(s.getRecursive())
		rec |= Recursive;

	if(s.getKDialogOptions())
		addMakeEnv("DIALOG=/usr/local/bin/kdialog-ports-wrapper");

	setRecursion(rec);
	setKeepGoing(s.getKeepGoing());
	setPackageUse((PackageUse)  s.getPackageUse());
	addMakeArgs(s.getMakeArguments());
	addMakeEnv(s.getMakeEnvironment());
}



/** Returns a QProcess that applies the specified
  * actions on the given list of packages or NULL
  * if not packages for upgrading were specified.
  *
  * The returned QProcess must be deleted after use.
  */
UnixProcess * PkgUpgradeEngine::getProcess() {
	/* Stop here when no packages are specified */
	if(packages.isEmpty())
		return  NULL;


	UnixProcess  *process = new UnixProcess(QString(PORTUPGRADE_CMD));
	process->setWorkingDirectory(QDir(PACKAGES_DIRECTORY));

	if(recursion & Recursive)
		process->addArgument("-r");
	if(recursion & UpwardRecursive)
		process->addArgument("-R");

	if(installNewPkgs)
		process->addArgument("-N");

	if(keepGoing)
		process->addArgument("-k");

	switch(packageUse) {
		case UsePackages:
			process->addArgument("-P");
			break;

		case UsePackagesOnly:
			process->addArgument("-PP");
			break;

		default: break;
	}

	if(!makeArgs.isEmpty()) {
		process->addArgument("-m");
		process->addArgument(makeArgs.join(" "));
	}

	if(!makeEnv.isEmpty()) {
		process->addArgument("-M");
		process->addArgument(makeEnv.join(" "));
	}

	Package  *pkg = packages.first();
	while(pkg != NULL) {
		process->addArgument(pkg->getID());

		pkg = packages.next();
	}

	process->setEnvironment(getEnvironment());
	return  process;
}


QString PkgUpgradeEngine::readDefaultRepository() {
	const struct {
		int from;     /* Lowest version number to match */
		int to;      /* Highest version number to match */
		const char *repository;  /* Directory it lives in */
	} releases[] = {
		{ 300000, 399000, "3-stable" },
		{ 400000, 499000, "4-stable" },
		{ 502100, 502128, "5-current" },
		{ 503100, 599000, "5-stable" },
		{ 600100, 699000, "6-stable" },
		{ 700000, 799000, "7-stable" },
		{ 0, 9999999, "current" },
		{ 0, 0, NULL }
	};

	int release = getosreldate();
	for(int i = 0; releases[i].repository != NULL; ++i)
		if(release >= releases[i].from && release <= releases[i].to)
			return releases[i].repository;

	return "";
}


QStringList PkgUpgradeEngine::getEnvironment() {
	/* Find out architecture */
	struct utsname u;
	uname(&u);

	/* Server package directory */
	QString packageDirectory = PackageSettings().getPackageDirectory();
	
	QString site = "PACKAGESITE=";
	if(packageDirectory.startsWith("ftp://") || packageDirectory.startsWith("http://"))
		site += packageDirectory;
	else {
		site += QString("ftp://packageftp.desktopbsd.net/pub/FreeBSD/ports/%1/packages-%2/Latest/")
						.arg(u.machine)
						.arg(packageDirectory.isEmpty() ? readDefaultRepository() : packageDirectory);
	}
				  
	return QStringList(site);
}
